import sys, time, math, os, random
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from Texture2D import *

startDir = os.getcwd()


class BlendFilter:
    def __init__(self, srcFunc=GL_ZERO, destFunc=GL_SRC_ALPHA, color=[1,1,1,1]):
        self.srcFunc = srcFunc
        self.destFunc = destFunc
        self.color = color

    def draw(self):
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        glMatrixMode(GL_MODELVIEW)
        
        glEnable(GL_BLEND)
        glBlendFunc(self.srcFunc, self.destFunc)
        glColor4fv(self.color)
        glBegin(GL_QUADS)
        glVertex2f(-1, -1)
        glVertex2f(-1, 1)
        glVertex2f(1, 1)
        glVertex2f(1, -1)
        glEnd()
        
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)


class TexAlphaSquare:
    def __init__(self, width=1.0, height=1.0, xpos=0.0, ypos=0.0, heading=0, texture=None):
        self.width = width
        self.height = height
        self.xpos = xpos
        self.ypos = ypos
        self.heading = heading
        self.texture = texture
    def draw(self):
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        if self.texture: self.texture.apply()
        glColor4f(1,1,1,1)
        glPushMatrix()
        glTranslatef(self.xpos, self.ypos, 0)
        glRotatef(self.heading, 0, 0, 1)
        glBegin(GL_QUADS)
        glTexCoord2f(0, 0)
        glVertex2f(-self.width/2.0, -self.height/2.0)
        glTexCoord2f(1, 0)
        glVertex2f(self.width/2.0, -self.height/2.0)
        glTexCoord2f(1, 1)
        glVertex2f(self.width/2.0, self.height/2.0)
        glTexCoord2f(0, 1)
        glVertex2f(-self.width/2.0, self.height/2.0)
        glEnd()
        glPopMatrix()
        if self.texture: self.texture.disable()
        glDisable(GL_BLEND)

objects = []
thefilter = None

def createObjects():
    global objects, thefilter
    bg = TexAlphaSquare(10, 10, texture=Texture2D('hst9904a.jpg'))
    objects.append(bg)
    rocket = TexAlphaSquare(3, 3, xpos=3, ypos=0, texture=Texture2D('rocket.tif'))
    objects.append(rocket)
    thefilter = BlendFilter(GL_ZERO, GL_SRC_ALPHA)


def draw():
    glClearColor(0, 0.3, 0.5, 0)
    glClear(GL_COLOR_BUFFER_BIT)
    
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluOrtho2D(-5, 5, -5, 5)
    glMatrixMode(GL_MODELVIEW)

    for o in objects:
        o.draw()
    thefilter.draw()
    
    glutSwapBuffers()


def keyboard(key, x, y):
    if key == chr(27):
        sys.exit(0)

def specialkey(key,x,y):
    global objects
    if key == GLUT_KEY_LEFT:
        objects[0].xpos -= 0.25
    elif key == GLUT_KEY_RIGHT:
        objects[0].xpos += 0.25
    elif key == GLUT_KEY_UP:
        objects[0].ypos += 0.25
    elif key == GLUT_KEY_DOWN:
        objects[0].ypos -= 0.25


def update(dummy):
    glutTimerFunc(16, update, 0)
    global objects
    angle = (time.time() % 10) * 36
    objects[1].xpos = 3 * math.sin(angle * math.pi/180.0)
    objects[1].ypos = 3 * math.cos(angle * math.pi/180.0)
    objects[1].heading = 270 - angle
    thefilter.color[3] = (1.0+math.sin(time.time()))/2.0
    glutPostRedisplay()

glutInit([])
glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE)
glutInitWindowSize(400,400)
glutCreateWindow(sys.argv[0])
glutDisplayFunc(draw)
glutKeyboardFunc(keyboard)
glutSpecialFunc(specialkey)
glutTimerFunc(1, update, 0)
os.chdir(startDir)
createObjects()
glutMainLoop()
